package app.login;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import app.error.CustomAppException;
import app.saaspass.SaaspassApiService;
import app.saaspass.error.RestApiException;
import app.saaspass.error.SaaspassRestErrorEnum;
import app.security.JWTToken;
import app.security.TokenProvider;

/**
 * Service for manual login.
 */
@Service
public class LoginService {

	private static final Logger log = LoggerFactory.getLogger(LoginService.class);

	@Autowired
	private SaaspassApiService saaspassApiService;

	@Autowired
	private TokenProvider tokenProvider;

	/**
	 * Example of user login authenticated by SAASPASS.
	 * @param username - username of the application account
	 * @param otp - One Time PAssword
	 * @return login token
	 */
	public JWTToken doLogin(String username, String otp) {
	
		if (!StringUtils.hasText(username)) {
			throw new CustomAppException("Invalid Username!");
		}
		if (!StringUtils.hasText(otp)) {
			throw new CustomAppException("Invalid OTP!");
		}
		saaspassApiService.saaspasssAuthentication(username, otp);
		String token = tokenProvider.createAdminToken(username, true);
	
		return new JWTToken(token);
	
	}

	/**
	 * Example of login with password and integration of SAASPASS. In this scenario
	 * if user is required 2fa with SAASPASS it will require One-Time Password as
	 * second step. Partial login with application credentials is performed.
	 * 
	 * @param username
	 * @param password
	 * @return login token
	 */
	public JWTToken doLoginWithPassword(String username, String password) {
		doPasswordCheck(username, password); // do your credential check
		String token;
		try {
			// Check with SAASPASS if user is allowed to login without One-Time Password
			// Define these rules under 
			// manage application > user authentication management > customize user authentication
			//https://developers.saaspass.com/#one-time-password-settings
			saaspassApiService.saaspasssAuthentication(username, "");
			token = tokenProvider.createAdminToken(username, false);
		} catch (RestApiException e) {
			if (SaaspassRestErrorEnum.LOGINFAIL_OTP_MANDATORY_FOR_ACCOUNT.name()
					.equals(e.getErrorResponse().getName())) {
				token = tokenProvider.createPartialToken(username);
			} else {
				throw e;
			}
		}
		// login user
		return new JWTToken(token);

	}

	/**
	 * Example of login with password and integration of SAASPASS. In this scenario
	 * password and One-Time Password are provided at once.
	 * If One-Time Password is mandatory can be customized
	 * https://developers.saaspass.com/#one-time-password-settings
	 * @param username
	 * @param password
	 * @param otp
	 * @return login token
	 */
	public JWTToken doLoginWithPassword(String username, String password, String otp) {

		doPasswordCheck(username, password);
		saaspassApiService.saaspasssAuthentication(username, otp);
		String token = tokenProvider.createAdminToken(username, true);
		return new JWTToken(token);

	}

	/**
	 * Application's custom authentication.
	 * 
	 * @param username
	 * @param password
	 */
	private void doPasswordCheck(String username, String password) {
		if (!StringUtils.hasText(username)) {
			throw new CustomAppException("Invalid Username!");
		}

		String userPassword = UserUtil.getUserPasswordByUsername(username);
		if (userPassword == null) {
			throw new CustomAppException("User doesn't exists!");
		}
		if (!userPassword.equals(password)) {
			throw new CustomAppException("Invalid Credentials!");
		}

	}

}
