package app.registration;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import app.error.CustomAppException;
import app.login.UserUtil;
import app.saaspass.SaaspassApiService;
import app.saaspass.SpAccountResponse;
import app.security.JWTToken;
import app.security.TokenProvider;

@Service
public class RegistrationService {
	
	@Autowired
    private SaaspassApiService saaspassApiService;

    @Autowired
    private SimpMessagingTemplate simpMessagingTemplate;

    @Autowired
    private TokenProvider tokenProvider;
    
    /**
     * Method showing simple IR flow with just username and SAASPASS.
     * Depending on application needs at this point can be used other fields from registration profile
     * or password provided by user.
     * @param username - username for the new account
     * @param session - session for reverse ajax
     * @param tracker - tracker for validation with SAASPASS
     */
    public void instantRegistration(String username, String session, String tracker) {
    	
        saaspassApiService.checkInstantLoginTracker(username, tracker);
        registerUser(username,null); //do your application registration 
        String destination = "/topic/instant-registration/" + session;
        //login user after registration
        JWTToken payload = new JWTToken(tokenProvider.createAdminToken(username,true));
        simpMessagingTemplate.convertAndSend(destination, payload);
    }
    
    private void  registerUser(String username,String password) {
    	if(password==null) {
    		password="1234"; //setting default password
    	}
    	String existingUserPassword=UserUtil.getUserPasswordByUsername(username);
    	if(existingUserPassword!=null) {
    		throw new CustomAppException("User already exists!");
    	}
    	
    	UserUtil.addUser(username, password);
    }
    
    public void registerUserWithPassword(String username,String password) {
    	if (!StringUtils.hasText(username)) {
			throw new CustomAppException("Username cannot be empty!");
		}
    	if (!StringUtils.hasText(password)) {
			throw new CustomAppException("Password cannot be empty!");
		}
    	registerUser(username, password);
    }
    
    public SpAccountResponse registerUserWithSaaspass(String username,String saaspassId,String otp) {
    	if (!StringUtils.hasText(username)) {
			throw new CustomAppException("Username cannot be empty!");
		}
    	if (!StringUtils.hasText(otp)) {
			throw new CustomAppException("Invalid OTP!");
		}
    	SpAccountResponse data = saaspassApiService.addApplicationAccountByUser(username, otp, saaspassId,false);
    	registerUser(username, null);
    	return data;
    }
    
    public SpAccountResponse registerUserWithSaaspassAndPassword(String username,String password,String saaspassId,String otp) {
    	if (!StringUtils.hasText(username)) {
			throw new CustomAppException("Username cannot be empty!");
		}
    	if (!StringUtils.hasText(password)) {
			throw new CustomAppException("Password cannot be empty!");
		}
    	if (!StringUtils.hasText(otp)) {
			throw new CustomAppException("Invalid OTP!");
		}
    	SpAccountResponse data = saaspassApiService.addApplicationAccountByUser(username, otp, saaspassId,false);
    	registerUser(username, password);
    	return data;
    }
    
    public SpAccountResponse registerUserByAdmin(String username,String accountOwner,Boolean isAdAccount) {
    	if (!StringUtils.hasText(username)) {
			throw new CustomAppException("Username cannot be empty!");
		}
    	SpAccountResponse data = saaspassApiService.addApplicationAccountByAdmin(username, accountOwner, (isAdAccount!=null && isAdAccount));
    	registerUser(username,null);
    	return data;
    }
    
    public Object registerAndLoginUser(String username,String password,String saaspassId,String otp) {
    	SpAccountResponse	registeredAccount=registerUserWithSaaspassAndPassword(username, password, saaspassId, otp);
    	JWTToken token=getLoginTokenForNewlyRegistered(username, registeredAccount);
    	if(token==null) {
    		return registeredAccount;
    	}
    	return token;
    }
    public Object registerAndLoginUser(String username,String saaspassId,String otp) {
    	SpAccountResponse	registeredAccount=registerUserWithSaaspass(username,  saaspassId, otp);
    	JWTToken token=getLoginTokenForNewlyRegistered(username, registeredAccount);
    	if(token==null) {
    		return registeredAccount;
    	}
    	return token;
    }
    public JWTToken getLoginTokenForNewlyRegistered(String username,SpAccountResponse registeredAccount) {
    	if(registeredAccount.getIsVerified() && !registeredAccount.getIsAccountOwnerBlocked() && (registeredAccount.getWarning()==null || registeredAccount.getWarning().isEmpty())) {
			String token = tokenProvider.createAdminToken(username, true);
			return new JWTToken(token);
		}
    	return null;
    }
    
    public SpAccountResponse unregisterUser(String username) {
    	UserUtil.removeUser(username);
    	SpAccountResponse data = saaspassApiService.unregisterAccount(username);
    	return data;
    }

}
