package app.saaspass;

import static org.springframework.http.HttpStatus.FORBIDDEN;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import app.barcode.BarcodeResponse;
import app.saaspass.error.RestApiException;
import app.saaspass.error.SaaspassRestErrorEnum;
import app.saaspass.token.ApplicationTokenService;
import app.saaspass.token.CompanyTokenService;

@Service
public class SaaspassApiService {

	@Autowired
	private RestApiUlrProvider restApiUlrProvider;

	@Autowired
	@Qualifier("saaspassRestTemplate")
	private RestTemplate restTemplate;

	@Autowired
	private ApplicationTokenService applicationTokenService;

	@Autowired
	private CompanyTokenService companyTokenService;

	/**
	 * See <a href="https://developers.saaspass.com/#scan-barcode">https://developers.saaspass.com/#scan-barcode</a>
	 * and <a href="https://developers.saaspass.com/#proximity-login-instant-login">https://developers.saaspass.com/#proximity-login-instant-login</a>
	 */
	public BarcodeResponse getBarcode(BarcodeType barcodeType, String session) {
		return getBarcode(barcodeType, session, false);
	}

	
	private BarcodeResponse getBarcode(BarcodeType barcodeType, String session, boolean refreshToken) {
		String token = applicationTokenService.getToken();
		if (refreshToken) {
			token = applicationTokenService.getNewToken();
		}
		try {
			String url = restApiUlrProvider.getBarcodeUrl(barcodeType, session, token);
			BarcodeResponse data = restTemplate.getForObject(url, BarcodeResponse.class);
			return data;
		} catch (RestApiException e) {
			if (!refreshToken && e.getStatusCode() == FORBIDDEN
					&& SaaspassRestErrorEnum.EXPIRED_TOKEN.name().equals(e.getErrorResponse().getName())) {
				return getBarcode(barcodeType, session, true);
			}
			throw e;
		}
	}

	/**
	 * See <a href="https://developers.saaspass.com/#push-login-instant-login">https://developers.saaspass.com/#push-login-instant-login</a>
	 */
	public void sendPush(String username, String session) {
		String token = applicationTokenService.getToken();
		String url = restApiUlrProvider.getPushUrl(username, session, token);
		restTemplate.getForObject(url, Void.class);
	}

	/**
	 * If otp is provided it will check authentication with SAASPASS, if otp is empty string
	 * will check if user is allowed to login based on settings under
	 * manage application > user authentication management > customize user authentication
	 * more details: 
	 * <a href="https://developers.saaspass.com/#one-time-password">https://developers.saaspass.com/#one-time-password</a>
	 */
	public void saaspasssAuthentication(String username, String otp) {
		String token = applicationTokenService.getToken();
		String url = restApiUlrProvider.getOtpCheckUrl(username, otp, token);
		restTemplate.getForObject(url, Void.class);
	}

	/**
	 * Validates request from SAASPASS (Barcode Scan, SSO, Push, Proximity)
	 * See <a href="https://developers.saaspass.com/#tracker-id-validation">https://developers.saaspass.com/#tracker-id-validation</a>
	 */
	public void checkInstantLoginTracker(String username, String tracker) {
		String token = applicationTokenService.getToken();
		String url = restApiUlrProvider.getInstantLoginTrackerUrl(username, tracker, token);
		restTemplate.getForObject(url, Void.class);
	}

	/**
	 * See <a href="https://developers.saaspass.com/#register-account-by-user">https://developers.saaspass.com/#register-account-by-user</a>
	 */
	public SpAccountResponse addApplicationAccountByUser(String username, String otp, String accountOwner,
			boolean isAdAccount) {
		String token = applicationTokenService.getToken();
		String url = restApiUlrProvider.getAddApplicationAccountByUserUrl(username, otp, accountOwner, isAdAccount,
				token);
		SpAccountResponse data = restTemplate.getForObject(url, SpAccountResponse.class);
		return data;
	}

	 /**
	 * See <a href="https://developers.saaspass.com/#register-account-by-admin">https://developers.saaspass.com/#register-account-by-admin</a>
	 */
	public SpAccountResponse addApplicationAccountByAdmin(String username, String accountOwner, boolean isAdAccount) {
		String token = applicationTokenService.getToken();
		String url = restApiUlrProvider.getAddAccountToApplicationByAdminUrl(username, accountOwner, isAdAccount,
				token);
		SpAccountResponse data = restTemplate.getForObject(url, SpAccountResponse.class);
		return data;
	}

	 /**
	 * See <a href="https://developers.saaspass.com/#unregister-account">https://developers.saaspass.com/#unregister-account</a>
	 */
	public SpAccountResponse unregisterAccount(String username) {
		String token = companyTokenService.getToken();
		String url = restApiUlrProvider.getUnregisterAccountUrl(username, token);
		SpAccountResponse data = restTemplate.getForObject(url, SpAccountResponse.class);
		return data;
	}

	/**
	 * See <a href="https://developers.saaspass.com/#add-account">https://developers.saaspass.com/#add-account</a>
	 * @param username - username of account to be added in SAASPASS
	 * @param groupList - comma separated names of groups in SAASPASS (optional)
	 * @param accountOwner - identifier for SAASPASS USER (optional)
	 * @return - {@link SpAccountResponse}
	 */
	public SpAccountResponse addCompanyAccount(String username, String groupList, String accountOwner) {
		String token = companyTokenService.getToken();
		String url = restApiUlrProvider.getAddCompanyAccountUrl(username, groupList, accountOwner, token);
		SpAccountResponse data = restTemplate.getForObject(url, SpAccountResponse.class);
		return data;

	}

	 /**
	 * See <a href="https://developers.saaspass.com/#remove-account">https://developers.saaspass.com/#remove-account</a>
	 */
	public SpAccountResponse removeAccount(String username) {
		String token = companyTokenService.getToken();
		String url = restApiUlrProvider.getRemoveAccountUrl(username, token);
		SpAccountResponse data = restTemplate.getForObject(url, SpAccountResponse.class);
		return data;
	}

	 /**
	 * See <a href="https://developers.saaspass.com/#verify-account">https://developers.saaspass.com/#verify-account</a>
	 */
	public SpAccountResponse verifyAccount(String username, String accountOwner, Boolean isAdAccount) {
		String token = companyTokenService.getToken();
		String url = restApiUlrProvider.getVeryfyAccountUrl(username, accountOwner, isAdAccount, token);
		SpAccountResponse data = restTemplate.getForObject(url, SpAccountResponse.class);
		return data;
	}

}
