package app.security;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.MalformedJwtException;
import io.jsonwebtoken.SignatureAlgorithm;
import io.jsonwebtoken.SignatureException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.stream.Collectors;

@Component
public class TokenProvider {

    private final Logger log = LoggerFactory.getLogger(TokenProvider.class);

    private static final String AUTHORITIES_KEY = "auth";

    @Value("${security.secret}")
    private String secretKey;

    @Value("${security.token-validity-in-minutes}")
    private Long tokenValidityInMinutes;

    public String createAdminToken(String username,boolean saaspassAuthorized) {

        long now = new Date().getTime();
        long tokenValidityInMilliseconds = tokenValidityInMinutes * 60 * 1000;
        Date validity = new Date(now + tokenValidityInMilliseconds);

        String roles= "ROLE_ADMIN";
        if(saaspassAuthorized) {
        	roles="ROLE_ADMIN,ROLE_SAASPASS";
        }
		return Jwts.builder()
            .setSubject(username)
            .claim(AUTHORITIES_KEY, roles)
            .signWith(SignatureAlgorithm.HS512, secretKey)
            .setExpiration(validity)
            .compact();
    }
    public String createPartialToken(String username) {

        long now = new Date().getTime();
        long tokenValidityInMilliseconds = tokenValidityInMinutes * 60 * 1000;
        Date validity = new Date(now + tokenValidityInMilliseconds);

        return Jwts.builder()
            .setSubject(username)
            .claim(AUTHORITIES_KEY, "ROLE_PARTIAL")
            .signWith(SignatureAlgorithm.HS512, secretKey)
            .setExpiration(validity)
            .compact();
    }

    public Authentication getAuthentication(String token) {
        Claims claims = Jwts.parser()
            .setSigningKey(secretKey)
            .parseClaimsJws(token)
            .getBody();

        Collection<? extends GrantedAuthority> authorities =
            Arrays.stream(claims.get(AUTHORITIES_KEY).toString().split(","))
                .map(SimpleGrantedAuthority::new)
                .collect(Collectors.toList());

        User principal = new User(claims.getSubject(), "", authorities);

        return new UsernamePasswordAuthenticationToken(principal, "", authorities);
    }

    public boolean validateToken(String authToken) {
        try {
            Jwts.parser().setSigningKey(secretKey).parseClaimsJws(authToken);
            return true;
        } catch (SignatureException | MalformedJwtException e) {
            log.info("Invalid JWT signature: " + e.getMessage());
            return false;
        }
    }
}
